package ld20.behavior
{
    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import ld20.Embedded;
	import stego.behavior.Base;
    import stego.constants.Colors;
    import stego.constants.Direction;
    import stego.GameObject;
    import stego.util.graphics.AnimatedSpriteSheet;
    import stego.util.GraphicsUtil;
	
    public class PlayerGraphics extends Base
    {
        protected var tdm:TopDownMover;
        protected var spriteSheet:AnimatedSpriteSheet;

        public function get attached():BitmapData { return _attached; }
        protected var _attached:BitmapData;

        public function get attachedBmp():Bitmap { return _attachedBmp; }
        protected var _attachedBmp:Bitmap;

        public function PlayerGraphics(tdm:TopDownMover)
        {
            super(null);
            this.tdm = tdm;
        }

        override public function init(character:GameObject):void
        {
            super.init(character);

            var bmp:Bitmap = new Bitmap;
            _character.addChild(bmp);
            spriteSheet = new AnimatedSpriteSheet(bmp, Embedded.sprites_character, 16, 16);
            bmp.x = -8;
            bmp.y = -8;
            spriteSheet.addAnimation("idle_right", [0]);
            spriteSheet.addAnimation("idle_left", [2]);
            spriteSheet.addAnimation("idle_up", [6]);
            spriteSheet.addAnimation("idle_down", [4]);
            spriteSheet.addAnimation("right", [1, 0]);
            spriteSheet.addAnimation("left", [3, 2]);
            spriteSheet.addAnimation("up", [7, 6]);
            spriteSheet.addAnimation("down", [5, 4]);
            spriteSheet.playAnimation("idle_down");
            _attachedBmp = new Bitmap(null);
            _character.addChild(_attachedBmp);
        }

        override public function shutdown():void
        {
            GraphicsUtil.clearSprite(_character);
            tdm = null;
            spriteSheet = null;
            super.shutdown();
        }

        override public function update(deltaTime:Number):void
        {
            super.update(deltaTime);

            var prev:String = spriteSheet.currentAnim;

            switch (tdm.currentDirection)
            {
                case Direction.UP:
                    spriteSheet.switchToAnimation("up");
                    _attachedBmp.x = 0 - 0.5 * _attachedBmp.width;
                    _attachedBmp.y = -8 - 0.5 * _attachedBmp.height;
                    break;
                case Direction.DOWN:
                    spriteSheet.switchToAnimation("down");
                    _attachedBmp.x = 0 - 0.5 * _attachedBmp.width;
                    _attachedBmp.y = 8 - 0.5 * _attachedBmp.height;
                    break;
                case Direction.LEFT:
                    spriteSheet.switchToAnimation("left");
                    _attachedBmp.x = -8 - 0.5 * _attachedBmp.width;
                    _attachedBmp.y = 0 - 0.5 * _attachedBmp.height;
                    break;
                case Direction.RIGHT:
                    spriteSheet.switchToAnimation("right");
                    _attachedBmp.x = 8 - 0.5 * _attachedBmp.width;
                    _attachedBmp.y = 0 - 0.5 * _attachedBmp.height;
                    break;
                case Direction.STOP:
                    if (0 != prev.indexOf("idle_"))
                    {
                        spriteSheet.switchToAnimation("idle_"+prev);
                    }
                    break;
                default:
                    throw new Error("unrecognized direction \""+tdm.currentDirection+"\"");
                    break;
            }

            spriteSheet.update(deltaTime);
        }

        public function attach(bmpData:BitmapData):void
        {
           _attachedBmp.bitmapData = bmpData;
           _attached = bmpData;
        }
    }
}