package ld20.behavior
{
	import stego.behavior.Base;
    import stego.constants.Direction;
    import stego.constants.Keyboard;
    import stego.GameObject;
	import stego.util.InputUtil;
    import stego.util.MathUtil;
	
    /** behavior for characters with four-way movement */
    public class TopDownMover extends Base
    {
        public var moveUpKeys:Array = [Keyboard.UP, Keyboard.W];
        public var moveDownKeys:Array = [Keyboard.DOWN, Keyboard.S];
        public var moveRightKeys:Array = [Keyboard.RIGHT, Keyboard.D];
        public var moveLeftKeys:Array = [Keyboard.LEFT, Keyboard.A];

        /** How quickly you slow down when no keys are pressed */
        public var stopAccel:Number = 500;

        /** How quickly you speed up */
        public var runAccel:Number = 500;

        /** max movement speed */
        public var maxSpeed:Number = 125;

        /** current direction */
        public var currentDirection:uint;

        /** which key is currently being used for walking */
        public var walkKey:uint;

        public function TopDownMover(inputUtil:InputUtil)
        {
            super(inputUtil);
        }

        override public function init(character:GameObject):void
        {
            super.init(character);
            walkKey = 0;
        }

        override public function update(deltaTime:Number):void
        {
            super.update(deltaTime);
            doAccel(walkingControls(deltaTime), deltaTime);
        }

        /**
         * controls for moving left and right
         */
        protected function walkingControls(deltaTime:Number):uint
        {
            if ( (currentDirection != Direction.STOP) &&
                 (inputUtil.isDown(walkKey)) )
            {
                return currentDirection;
            }
            else
            {
                currentDirection = Direction.STOP;
                var key:uint;
                for each (key in moveLeftKeys)
                {
                    if (_inputUtil.isDown(key))
                    {
                        currentDirection = Direction.LEFT;
                        walkKey = key;
                        return currentDirection;
                    }
                }
                for each (key in moveRightKeys)
                {
                    if (_inputUtil.isDown(key))
                    {
                        currentDirection = Direction.RIGHT;
                        walkKey = key;
                        return currentDirection;
                    }
                }
                for each (key in moveUpKeys)
                {
                    if (_inputUtil.isDown(key))
                    {
                        currentDirection = Direction.UP;
                        walkKey = key;
                        return currentDirection;
                    }
                }
                for each (key in moveDownKeys)
                {
                    if (_inputUtil.isDown(key))
                    {
                        currentDirection = Direction.DOWN;
                        walkKey = key;
                        return currentDirection;
                    }
                }
            }
            return currentDirection;
        }

        /**
         * Handle the math to do the right kinds of acceleration
         * for platform character movement
         */
        protected function doAccel(desiredDirection:Number, deltaTime:Number):void
        {
            var directionMultiplier:Number = 0;
            switch (desiredDirection)
            {
                case Direction.LEFT:
                case Direction.UP:
                    directionMultiplier = -1;
                    break;
                case Direction.RIGHT:
                case Direction.DOWN:
                    directionMultiplier = 1;
                    break;
                case Direction.STOP:
                default:
                    directionMultiplier = 0;
                    break;
            }

            if ( (currentDirection == Direction.RIGHT) || (desiredDirection == Direction.LEFT) )
            {
                _character.dx = MathUtil.moveToward( character.dx,
                                                     directionMultiplier * maxSpeed,
                                                     (MathUtil.sameSign(directionMultiplier,_character.dx)?runAccel:stopAccel) * deltaTime );
                _character.dy = MathUtil.moveToward( character.dy,
                                                     0,
                                                     stopAccel * deltaTime );
            }
            // changing direction
            else if ( (currentDirection == Direction.DOWN) || (currentDirection == Direction.UP) )
            {
                _character.dx = MathUtil.moveToward( character.dx,
                                                     0,
                                                     stopAccel * deltaTime );
                _character.dy = MathUtil.moveToward( character.dy,
                                                     directionMultiplier * maxSpeed,
                                                     (MathUtil.sameSign(directionMultiplier,_character.dx)?runAccel:stopAccel) * deltaTime );
            }
            else // accelerating
            {
                _character.dx = MathUtil.moveToward( character.dx,
                                                     0,
                                                     stopAccel * deltaTime );
                _character.dy = MathUtil.moveToward( character.dy,
                                                     0,
                                                     stopAccel * deltaTime );
            }

            // if it gets over the max speed slow it down
            if (character.dx > maxSpeed)
            {
                character.dx = maxSpeed;
            }
            if (character.dx < -maxSpeed)
            {
                character.dx = -maxSpeed;
            }
            if (character.dy > maxSpeed)
            {
                character.dy = maxSpeed;
            }
            if (character.dy < -maxSpeed)
            {
                character.dy = -maxSpeed;
            }
        }
    }
}
