package ld20.gameobject
{
    import flash.display.Bitmap;
    import ld20.Embedded;
    import stego.constants.Direction;
    import stego.GameInterface;
	import stego.GameObject;
    import stego.util.collision.CollisionInfo;
    import stego.util.graphics.AnimatedSpriteSheet;
    import stego.util.GraphicsUtil;
    import stego.util.scene.GameObjectHolder;
	
	/**
     * ...
     * @author Michael Meyer
     */
    public class Enemy extends GameObject
    {
        public function get facingDirection():uint { return _facingDirection; }
        public function set facingDirection(value:uint):void
        {
            _facingDirection = value;
            switch (_facingDirection)
            {
                case Direction.RIGHT:
                    spriteSheet.switchToAnimation("right");
                    break;
                case Direction.LEFT:
                    spriteSheet.switchToAnimation("left");
                    break;
                case Direction.UP:
                    spriteSheet.switchToAnimation("up");
                    break;
                case Direction.DOWN:
                    spriteSheet.switchToAnimation("down");
                    break;
            }
        }
        protected var _facingDirection:uint = Direction.RIGHT;

        public var bulletLayer:GameObjectHolder;

        protected var spriteSheet:AnimatedSpriteSheet;

        protected static const SHOOT_PERIOD:Number = 1;
        protected static const bulletSpeed:Number = 100;
        protected var shootCountdown:Number;

        override public function init(game:GameInterface):void
        {
            super.init(game);

            if (numChildren == 0)
            {
                var bmp:Bitmap = new Bitmap;
                addChild(bmp);
                spriteSheet = new AnimatedSpriteSheet(bmp, Embedded.sprites_character, 16, 16);
                bmp.x = -8;
                bmp.y = -8;
                spriteSheet.addAnimation("right", [8]);
                spriteSheet.addAnimation("left", [9]);
                spriteSheet.addAnimation("down", [10]);
                spriteSheet.addAnimation("up", [11]);
                spriteSheet.playAnimation("right");
            }
            shootCountdown = 0;
        }

        override public function update(deltaTime:Number):void
        {
            super.update(deltaTime);
            spriteSheet.update(deltaTime);

            if (shootCountdown <= 0)
            {
                shoot();
                shootCountdown += SHOOT_PERIOD;
            }

            shootCountdown -= deltaTime;
        }

        protected function shoot():void
        {
            if (bulletLayer)
            {
                var bullet:GameObject = bulletLayer.add(Bullet) as Bullet;

                var nx:Number = 0,
                    ny:Number = 0;
                switch (_facingDirection)
                {
                    case Direction.UP:
                        ny = -1;
                        break;
                    case Direction.DOWN:
                        ny = 1;
                        break;
                    case Direction.LEFT:
                        nx = -1;
                        break;
                    case Direction.RIGHT:
                        nx = 1;
                        break;
                }
                bullet.dx = nx * bulletSpeed;
                bullet.dy = ny * bulletSpeed;
                bullet.x = x + 8 * nx;
                bullet.y = y + 8 * ny;
            }
        }

        override public function collisionOccurred(ci:CollisionInfo, object:GameObject):void
        {
            //super.collisionOccurred(ci, object);
            if (ci.hitbox2.userData.hasOwnProperty("nocollide")) { return; }

            if (ci.hitbox2.userData["item"] == Embedded.swordImage)
            {
                alive = false;
            }
        }
    }

}