package ld20.gameobject
{
    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import flash.geom.Rectangle;
    import ld20.behavior.PlayerGraphics;
    import ld20.behavior.TopDownMover;
    import ld20.Embedded;
    import stego.collections.LinkedList;
    import stego.GameInterface;
	import stego.GameObject;
    import stego.gameobject.TileMap;
    import stego.util.collision.CollisionInfo;
    import stego.util.collision.HitBox;
    import stego.util.MathUtil;
	
    public class Player extends GameObject
    {
        public function get attached():BitmapData { return pg.attached; }
        protected var pg:PlayerGraphics;

        override public function init(game:GameInterface):void
        {
            super.init(game);
            var tdm:TopDownMover = new TopDownMover(game.inputUtil);
            tdm.stopAccel = 2000;
            tdm.runAccel = 1000;
            tdm.maxSpeed = 75;
            addBehavior(tdm);
            pg = new PlayerGraphics(tdm);
            addBehavior(pg);
        }

        override public function canPassThrough(ci:CollisionInfo, object:GameObject):Boolean
        {
            var userdata:Object = ci.hitbox2.userData;
            return (userdata && userdata.hasOwnProperty("nocollide"));
        }

        /**
         * update the hitboxes array
         * Redoes it from scratch every frame using getBounds!
         * Override to be smarter and/or turn off autoUpdateHitboxes!
         */
        override protected function calculateHitboxes():void
        {
            var hb:HitBox;
            if (!hitboxes)
            {
                hitboxes = new LinkedList;
                hb = new HitBox();
                hb.x = -4;
                hb.y = 1;
                hb.width = 8;
                hb.height = 6;
                hb.userData = {item:"body"};
                hitboxes.push_back(hb);
                hb = new HitBox(0,0,0,0, {});
                hitboxes.push_back(hb);
            }

            hb = hitboxes.last.value;
            hb.userData["item"] = pg.attached;
            if (pg.attached == Embedded.swordImage)
            {
                hb.x = pg.attachedBmp.x + 6;
                hb.y = pg.attachedBmp.y;
                hb.width = pg.attachedBmp.width - 10;
                hb.height = pg.attachedBmp.height;
            }
            else if (pg.attached == Embedded.shieldImage)
            {
                hb.x = pg.attachedBmp.x + 2;
                hb.y = pg.attachedBmp.y + 4;
                hb.width = pg.attachedBmp.width - 4;
                hb.height = pg.attachedBmp.height - 5;
            }
            else
            {
                hb.x = hb.y = hb.width = hb.height = 0;
            }
        }

        override public function collisionOccurred(ci:CollisionInfo, object:GameObject):void
        {
            // main hitbox
            if (ci.hitbox1.userData["item"] == "body")
            {
                if ( (object is Player) &&
                     ((ci.hitbox2.userData["item"] == "body")) )
                {
                    if (object.alive && alive)
                    {
                        object.alive = false;
                        x = MathUtil.lerp(x, object.x, 0.5);
                        y = MathUtil.lerp(y, object.y, 0.5);
                        dx = dy = 0;
                    }
                }
                else if (object is Enemy)
                {
                    alive = false;
                }
                else
                {
                    super.collisionOccurred(ci, object);
                }
            }
            else if ( (ci.hitbox1.userData["item"] == Embedded.shieldImage) &&
                      (object is Enemy) )
            {
                super.collisionOccurred(ci, object);
            }
        }

        public function attach(bmpData:BitmapData):void
        {
            pg.attach(bmpData);
        }
    }
}



