package ld20.rooms
{
    import flash.display.BitmapData;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import ld20.Embedded;
    import ld20.gameobject.Enemy;
    import ld20.gameobject.ShieldSource;
    import ld20.gameobject.SwordSource;
    import stego.constants.Direction;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.gameobject.TileMap;
    import stego.ui.bitmapfont.BitmapFontTextField;
    import stego.util.scene.GameObjectHolder;
    import stego.util.scene.Scene;

    public class RoomLoader
    {
        public static const SPAWNTYPE_ONE_AT_A_TIME:String = "one at a time";
        public static const SPAWNTYPE_CONTINUOUS:String = "continuous";

        public var scene:Scene;
        public var platformLayer:GameObjectHolder;
        public var itemLayer:GameObjectHolder;
        public var playerLayer:GameObjectHolder;
        public var enemyLayer:GameObjectHolder;
        public var uiLayer:GameObjectHolder;
        public var currentRoom:int;
        public var spawnPoint:Point = new Point();
        public var spawnArea:Rectangle = new Rectangle();
        public var spawnType:String = SPAWNTYPE_ONE_AT_A_TIME;
        public var tileMap:TileMap;

        /** get it ready for use */
        public function init(game:GameInterface):void
        {
            scene = game.layers;
            this.platformLayer = scene.addLayer();
            this.itemLayer = scene.addLayer();
            this.enemyLayer = scene.addLayer();
            this.playerLayer = scene.addLayer();
            this.uiLayer = scene.addLayer();
            currentRoom = 0;
        }

        /** clean up */
        public function shutdown():void
        {
            unLoadRoom();
            this.platformLayer = null;
            this.itemLayer = null;
            this.playerLayer = null;
            this.enemyLayer = null;
            this.uiLayer = null;
        }

        /** clean up everything we loaded */
        public function unLoadRoom():void
        {
            platformLayer.removeAll();
            itemLayer.removeAll();
            playerLayer.removeAll();
            enemyLayer.removeAll();
            uiLayer.removeAll();
        }

        /** load next room */
        public function loadNextRoom():void
        {
            loadRoom((++currentRoom % RoomData.ROOMS.length));
        }

        /** load the given room */
        public function loadRoom(roomIndex:int):void
        {
            unLoadRoom();

            var map:XML = RoomData.ROOMS[roomIndex],
                firstGid:int = map.tileset[0].@firstgid,
                mapWidth:Number = map.@width,
                mapHeight:Number = map.@height,
                layer:XML = map.layer[0],
                layerWidth:int = layer.@width,
                layerHeight:int = layer.@height,
                tileWidth:int = int(map.@tilewidth),
                tileHeight:int = int(map.@tileheight),
                tileset:XML = map.tileset[0],
                tilesetImage:BitmapData = Embedded.rooms_tileset,
                tilesetWidth:int = tilesetImage.width,
                tilesetHeight:int = tilesetImage.height,
                mapRowsPerTile:int = Math.ceil(tilesetHeight / tileHeight),
                mapColsPerTile:int = Math.ceil(tilesetWidth / tileWidth),
                mapRow:int = 0,
                mapCol:int = 0,
                tile:XML,
                tilesetUserData:Array = [],
                index:int;
            this.currentRoom = roomIndex;
            tileMap = platformLayer.add(TileMap) as TileMap;
            tileMap.tilesetImage = tilesetImage;
            tileMap.tileWidth = tileWidth;
            tileMap.tileHeight = tileHeight;
            tileMap.width = mapWidth * tileWidth;
            tileMap.height = mapHeight * tileHeight;

            for each (tile in tileset.tile)
            {
                for each (var property:XML in tile.properties.property)
                {
                    var userdata:Object = tilesetUserData[int(tile.@id)];
                    if (!userdata)
                    {
                        userdata = tilesetUserData[int(tile.@id)] = {};
                    }
                    userdata[property.@name.toString().toLowerCase()] = property.@value.toString().toLowerCase();
                }
            }

            for each (tile in layer.data.tile)
            {
                index = tile.@gid - firstGid;
                if (index >= 0)
                {
                    setTile(tileMap, mapCol, mapRow, index, tilesetUserData[index]);
                }

                ++mapCol;
                if (mapCol >= layerWidth)
                {
                    mapCol %= layerWidth;
                    ++mapRow;
                }
            }
        }

        /** check the userdata before setting the tile, because it may actually use a different index */
        protected function setTile( tileMap:TileMap, col:uint, row:uint, index:int, userdata:Object ):void
        {
            var tileUserData:Object = { };
            var displayIndex:int = index;

            for (var propname:String in userdata)
            {
                switch(propname)
                {
                    case "nocollide":
                        tileUserData["nocollide"] = true;
                        break;
                    case "exit":
                        tileUserData["exit"] = index;
                        break;
                    case "spawn":
                        spawnPoint.x = tileMap.x + (col + 0.5) * tileMap.tileWidth;
                        spawnPoint.y = tileMap.y + (row + 0.5) * tileMap.tileHeight;
                        spawnArea.x = spawnPoint.x - 8;
                        spawnArea.y = spawnPoint.y - 8;
                        spawnArea.width = 16;
                        spawnArea.height = 16;
                        spawnType = userdata[propname];
                        break;
                    case "usetile":
                        displayIndex = userdata[propname];
                        break;
                    case "showtext":
                        var bftf:BitmapFontTextField = uiLayer.add(BitmapFontTextField) as BitmapFontTextField;
                        bftf.x = tileMap.x + col * (tileMap.tileWidth + 0.5);
                        bftf.y = tileMap.y + row * (tileMap.tileHeight + 0.5);
                        bftf.text = userdata[propname];
                        bftf.hAlign = BitmapFontTextField.HALIGN_CENTER;
                        bftf.vAlign = BitmapFontTextField.VALIGN_MIDDLE;
                        break;
                    case "enemy":
                        var enemy:Enemy = enemyLayer.add(Enemy) as Enemy;
                        enemy.x = tileMap.x + (col + 0.5) * tileMap.tileWidth;
                        enemy.y = tileMap.y + (row + 0.5) * tileMap.tileHeight;
                        enemy.bulletLayer = enemyLayer;
                        switch (userdata[propname])
                        {
                            case "right":
                                enemy.facingDirection = Direction.RIGHT;
                                break;
                            case "left":
                                enemy.facingDirection = Direction.LEFT;
                                break;
                            case "up":
                                enemy.facingDirection = Direction.UP;
                                break;
                            case "down":
                                enemy.facingDirection = Direction.DOWN;
                                break;
                            default:
                                throw new Error("which way it \""+userdata[propname]+"\"");
                                break;
                        }
                        break;
                    case "item":
                        switch (userdata[propname])
                        {
                            case "shield":
                                var shieldSource:GameObject = itemLayer.add(ShieldSource);
                                shieldSource.x = tileMap.x + (col + 0.5) * tileMap.tileWidth;
                                shieldSource.y = tileMap.y + (row + 0.5) * tileMap.tileHeight;
                                break;
                            case "sword":
                                var swordSource:GameObject = itemLayer.add(SwordSource);
                                swordSource.x = tileMap.x + (col + 0.5) * tileMap.tileWidth;
                                swordSource.y = tileMap.y + (row + 0.5) * tileMap.tileHeight;
                                break;
                            default:
                                throw new Error("unknown item \""+userdata[propname]+"\"");
                                break;
                        }
                        break;
                    default:
                        throw new Error("what do I do for property \""+propname+"\"?");
                        break;
                }
            }

            tileMap.setTile( col,
                             row,
                             displayIndex,
                             tileUserData );
        }
    }
}
