package ld20.ui
{
    import flash.display.Bitmap;
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import ld20.Embedded;
    import stego.GameInterface;
	import stego.GameObject;
    import stego.util.Easing;
    import stego.util.MathUtil;
	
    public class TitleScreenButton extends GameObject
    {
        protected var bitmap:Bitmap;
        public var callback:Function;
        protected var mouseEventDummy:Sprite;
        protected var grow:Boolean;
        protected var startScale:Number = 8;
        protected var endScale:Number = 200;
        protected var growTime:Number = 0.5;
        protected var growTimer:Number = 0;

        override public function init(game:GameInterface):void
        {
            super.init(game);

            if (null == mouseEventDummy)
            {
                mouseEventDummy = new Sprite();
                mouseEventDummy.graphics.beginFill(0, 0);
                mouseEventDummy.graphics.drawRect(0, 0, 1, 1);
                mouseEventDummy.graphics.endFill();
                this.addChild(mouseEventDummy);
            }
            if (!bitmap)
            {
                bitmap = new Bitmap(Embedded.swordImage);
                addChild(bitmap);
                bitmap.scaleX = bitmap.scaleY = startScale;
                reCenter();
                updateMouseEventDummy();
            }
            addMouseEventListeners();

            grow = false;
            growTimer = 0;
        }

        override public function update(deltaTime:Number):void
        {
            super.update(deltaTime);
            if (grow)
            {
                growTimer += deltaTime;

                var scale:Number = MathUtil.lerp( startScale,
                                                  endScale,
                                                  Easing.quadOut(growTimer / growTime) );
                bitmap.scaleX = bitmap.scaleY = scale;
                reCenter();

                if (growTimer >= growTime)
                {
                    if (null != callback)
                    {
                        callback();
                    }
                }
            }
        }

        protected function reCenter():void
        {
            bitmap.x = -0.5 * bitmap.width;
            bitmap.y = -0.5 * bitmap.height;
        }

        /**
         * make the dummy mouse event rect match the main graphic
         */
        protected function updateMouseEventDummy():void
        {
            var bounds:Rectangle = bitmap.getBounds(this);
            mouseEventDummy.x = bounds.x;
            mouseEventDummy.y = bounds.y;
            mouseEventDummy.width = bounds.width;
            mouseEventDummy.height = bounds.height;
        }

        /**
         * cleanup
         */
        public override function shutdown():void
        {
            callback = null;
            removeMouseEventListeners();
            super.shutdown();
        }

        private function addMouseEventListeners():void
        {
            if (mouseEventDummy)
            {
                mouseEventDummy.addEventListener(MouseEvent.ROLL_OVER, handleRollOver);
                mouseEventDummy.addEventListener(MouseEvent.ROLL_OUT, handleRollOut);
                mouseEventDummy.addEventListener(MouseEvent.CLICK, handleClick);
            }
        }

        private function removeMouseEventListeners():void
        {
            if (mouseEventDummy)
            {
                mouseEventDummy.removeEventListener(MouseEvent.ROLL_OVER, handleRollOver);
                mouseEventDummy.removeEventListener(MouseEvent.ROLL_OUT, handleRollOut);
                mouseEventDummy.removeEventListener(MouseEvent.CLICK, handleClick);
            }
        }

        /** call the callback when we get clicked */
        private function handleClick(e:MouseEvent):void
        {
            grow = true;
            removeMouseEventListeners();
            removeChild(mouseEventDummy);
            mouseEventDummy = null;
        }

        private function handleRollOver(evt:MouseEvent = null):void
        {
            bitmap.scaleX = bitmap.scaleY = startScale * 1.25;
            reCenter();
            updateMouseEventDummy();
        }

        private function handleRollOut(evt:MouseEvent = null):void
        {
            bitmap.scaleX = bitmap.scaleY = startScale;
            reCenter();
            updateMouseEventDummy();
        }
    }
}