import re
import os
import shutil

template_folder = 'new_project_template'
classname_re = re.compile('[A-Za-z]\w*$')

def get_file_contents(path):
    r"""return the contents of file"""
    f = open(path)
    contents = f.read()
    f.close()
    return contents
    
def write_file(path, contents):
    r"""dump contents into a file"""
    f = open(path, 'w')
    f.write(contents)
    f.close()
    
def project_name_valid(name):
    r"""check to see if that is a valid project name"""
    return None != classname_re.match(name)

def ask_for_project_name():
    r"""prompt the user for the name of their new project"""
    project_name = ''
    while True:
        project_name = raw_input('Enter a project name:')
        if project_name_valid(project_name):
            break
        else:
            print "project name should be a valid class name"
    return project_name

def ask_whether_to_do_hginit():
    r"""prompt the user for whether to create a mercurial repo for the new project"""
    while True:
        response = raw_input('Create Mercurial Repository? (Y/N):')
        if response[0].lower() == 'y':
            return True
        if response[0].lower() == 'n':
            return False

def copy_and_fill_in_template( template_file, new_path, **kwargs ):
    r"""copy a file and do the template replace to its contents"""
    contents = get_file_contents(template_file)
    contents = fill_in_template(contents, **kwargs)
    write_file(new_path, contents)

def fill_in_template( original, 
                      project_name, 
                      swf_name, 
                      package_name, 
                      main_class_name,
                      width,
                      height ):
    r"""Find magic template words and rplace them with the appropriate values"""
    original = original.replace('__PROJECTNAME__', project_name)
    original = original.replace('__SWFNAME__', project_name)
    original = original.replace('__PACKAGENAME__', package_name)
    original = original.replace('__MAINCLASSNAME__', project_name)
    original = original.replace('__WIDTH__', str(width))
    original = original.replace('__HEIGHT__', str(height))
    return original

def make_project(project_name, width, height):
    r"""copy and process files to make the new project"""
    package_name = project_name.lower()
    
    # figure out paths and names
    keyword_args = { 'project_name':project_name, 
                     'swf_name':project_name, 
                     'package_name':package_name, 
                     'main_class_name':project_name,
                     'width':width,
                     'height':height }
    project_path = os.path.join('..',project_name)
                         
    for root, dirs, files in os.walk(template_folder):
        for dir in dirs:
            source = os.path.join(root,dir)
            destination = source.replace(template_folder, project_path)
            destination = fill_in_template(destination, **keyword_args)
            if not os.path.exists(destination): os.makedirs(destination) 
        for file in files:
            source = os.path.join(root,file)
            destination = source.replace(template_folder, project_path)
            destination = fill_in_template(destination, **keyword_args)
            copy_and_fill_in_template(source, destination, **keyword_args)

def do_hg_init(project_name):
    r"""create a mercurial repo"""
    project_path = os.path.join('..', project_name)
    old_cwd = os.getcwd()
    os.chdir(project_path)
    os.system('hg init')
    write_hgignore_file()
    os.system('hg commit -A -u makenewproject.py -m "new project created"')
    os.chdir(old_cwd)

def write_hgignore_file():
    r"""create an .hgignore file for the newly created repo"""
    f = open('.hgignore','w')
    f.write('glob:obj/\nglob:bin/\n')
    f.close()

if __name__=="__main__":
    project_name = ask_for_project_name()
    make_project(project_name, 600, 500)
    if ask_whether_to_do_hginit():
        do_hg_init(project_name)

