﻿package stego.behavior.platformer
{
    import stego.behavior.Base;
    import stego.constants.Direction;
    import stego.constants.Keyboard;
    import stego.GameObject;
    import stego.util.InputUtil;
    import stego.util.MathUtil;

    /**
     * behavior for a player that can run left and right
     * @author Mike Meyer
     */
    public class Run extends Base
    {
        public var moveRightKeys:Array = [Keyboard.RIGHT, Keyboard.D];
        public var moveLeftKeys:Array = [Keyboard.LEFT, Keyboard.A];


        /** How quickly you slow down when no keys are pressed */
        public var stopAccel:Number = 500;

        /** How quickly you speed up when walking */
        public var runAccel:Number = 500;

        /** How quickly you speed up in the air */
        public var airAccel:Number = 187.5;

        /** How quickly you slow down in the air */
        public var airDecel:Number = 187.5;

        /** max running speed */
        public var maxRunSpeed:Number = 125;

        /** used when skidding while changing directions like in Super Mario Bros. */
        public var changeDirectionAccel:Number = 500;

        /**
         * constructor
         * @param	inputUtil
         */
        public function Run(inputUtil:InputUtil)
        {
            super(inputUtil);
        }

        /**
         * process one frame of controls
         */
        override public function update(deltaTime:Number):void
        {
            doRunAccel(walkingControls(deltaTime), deltaTime);
        }

        /**
         * controls for moving left and right
         */
        protected function walkingControls(deltaTime:Number):uint
        {
            var desiredDirection:Number = Direction.STOP;
            if (inputUtil.anyDown(moveRightKeys))
            {
                desiredDirection = Direction.RIGHT;
            }
            else if (inputUtil.anyDown(moveLeftKeys))
            {
                desiredDirection = Direction.LEFT;
            }
            return desiredDirection;
        }

        /**
         * Handle the math to do the right kinds of acceleration
         * for platform character movement
         */
        protected function doRunAccel(desiredDirection:Number, deltaTime:Number):void
        {
            var currentDirection:Number = Direction.STOP;
            if (character.dx > 0)
            {
                currentDirection = Direction.RIGHT;
            }
            else if (character.dx < 0)
            {
                currentDirection = Direction.LEFT;
            }

            var directionMultiplier:Number = 0;
            switch (desiredDirection)
            {
                case Direction.LEFT:
                    directionMultiplier = -1;
                    break;
                case Direction.RIGHT:
                    directionMultiplier = 1;
                    break;
                case Direction.STOP:
                default:
                    directionMultiplier = 0;
                    break;
            }

            // controls a bit differently on the ground
            if (character.isOnGround)
            {
                // stopping
                if ( (currentDirection != Direction.STOP) && (desiredDirection == Direction.STOP) )
                {
                    character.dx = MathUtil.moveToward(character.dx, directionMultiplier * maxRunSpeed, stopAccel * deltaTime);
                }
                // changing direction
                if ( (currentDirection != Direction.STOP) && (desiredDirection != currentDirection) )
                {
                    character.dx = MathUtil.moveToward(character.dx, directionMultiplier * maxRunSpeed, changeDirectionAccel * deltaTime);
                }
                else // accelerating
                {
                    character.dx = MathUtil.moveToward(character.dx, directionMultiplier * maxRunSpeed, runAccel * deltaTime);
                }
            }
            else
            {
                if (Direction.STOP == desiredDirection)
                {
                    character.dx = MathUtil.moveToward(character.dx, 0, airDecel * deltaTime);
                }
                else
                {
                    character.dx = MathUtil.moveToward(character.dx, directionMultiplier * maxRunSpeed, airAccel * deltaTime);
                }
            }

            // if it gets over the max speed slow it down
            if (character.dx > maxRunSpeed)
            {
                character.dx = maxRunSpeed;
            }
            if (character.dx < -maxRunSpeed)
            {
                character.dx = -maxRunSpeed;
            }

            // flip graphics
            if (desiredDirection > 0)
            {
                faceRight();
            }
            if (desiredDirection < 0)
            {
                faceLeft();
            }
        }

        /**
         * make the character face right
         */
        public function faceRight():void
        {
            _character.scaleX = Math.abs(_character.scaleX);
        }

        /**
         * make the character face left
         */
        public function faceLeft():void
        {
            _character.scaleX = -Math.abs(_character.scaleX);
        }
    }

}
