﻿package stego.script.step
{
    import stego.script.Step;

    /**
     * Step to loop a bunch of other steps N times
     * @author Mike Meyer
     */
    public class Loop extends Step
    {
        private var numLoops:Number;
        private var loopsLeft:Number;
        private var steps:/*Step*/Array;
        private var prevStep:Number;
        private var nextStep:Number;

        /**
         *
         * @param	numLoops - how many times to loop.  -1 to loop forever
         * @param	steps
         */
        public function Loop(numLoops:Number, steps:Array)
        {
            nextStep = 0;
            prevStep = -1;
            this.numLoops = numLoops;
            loopsLeft = numLoops;
            this.steps = steps.concat(); // make a copy
        }

        /**
         * Play the next step in the loop
         */
        public override function execute(deltaTime:Number):Boolean
        {
            // early out on empty steplists
            if (0 == steps.length)
            {
                return true;
            }

            var stepFinished:Boolean = true;
            // if the step finished, continue to the next one
            while (stepFinished && (loopsLeft != 0))
            {
                var step:Step = steps[nextStep];

                // if it is the first time we are running
                // it this loop, then we need to make sure
                // it loses any state information from the
                // last time through

                if (prevStep != nextStep)
                {
                    step.restart();
                }

                prevStep = nextStep;

                stepFinished = step.execute(deltaTime);
                if (stepFinished)
                {
                    ++nextStep;

                    if (nextStep >= steps.length)
                    {
                        prevStep = -1;
                        nextStep = 0;
                        if (loopsLeft > 0)
                        {
                            --loopsLeft;
                        }
                    }
                }
            }

            return loopsLeft == 0;
        }


        /**
         * clear out any state information the step might be saving and
         * go back to like when it was first created
         */
        public override function restart():void
        {
            for (var i:Number = 0; i < steps.length; ++i)
            {
                steps[i].restart();
            }

            loopsLeft = numLoops;
            nextStep = 0;
            prevStep = -1;
        }
    }

}
