﻿package stego.script.step
{
    import stego.script.Step;

    /**
     * step to combine multiple steps into one
     * (to try and execute all on the same update call)
     * @author Mike Meyer
     */
    public class Parallel extends Step
    {
        private var _steps:Array;
        private var _results:Array;

        public function Parallel(steps:Array)
        {
            _steps = steps.concat(); // make a copy
            _results = new Array();
        }

        public override function execute(deltaTime:Number):Boolean
        {
            var allReady:Boolean = true;

            for (var i:Number = 0; i < _steps.length; ++i)
            {
                var step:Step = _steps[i];

                // if it is the first time we are running
                // it, then we need to make sure
                // it loses any state information from the
                // last time through
                if (null == _results[i])
                {
                    step.restart();
                }

                if (true != _results[i])
                {
                    _results[i] = step.execute(deltaTime);

                    if (!_results[i])
                    {
                        allReady = false;
                    }
                }
            }

            // none of them blocked
            return allReady;
        }

        /**
         * Go back to the beginning
         */
        public override function restart():void
        {
            for (var i:Number = 0; i < _steps.length; ++i)
            {
                _steps[i].restart();
            }

            _results = new Array();
        }
    }

}
