﻿package stego.script.step
{
    import flash.events.Event;
    import flash.media.SoundChannel;
    import stego.script.Step;
    import flash.media.Sound;
    import stego.util.Sounds;

    /**
     * script step to start a sound and wait for it to finish
     */
    public class PlaySound extends Step
    {
        private var sound:Sound;
        private var channel:SoundChannel;
        private var soundComplete:Boolean;
        private var waitForSoundToFinish:Boolean;

        public function PlaySound(s:Sound, waitForSoundToFinish:Boolean = false)
        {
            sound = s;
            this.waitForSoundToFinish = waitForSoundToFinish;
            restart();
        }

        /**
         * play the sound or continue waiting for it to finish
         * @return true if finished, false if it needs to be run some more
         */
        public override function execute(deltaTime:Number):Boolean
        {
            if (null == channel)
            {
                channel = Sounds.playSound(sound, null);
                if (null == channel)
                {
                    // well that sucks. sound isn't working :(
                    soundComplete = true;
                }
                else
                {
                    channel.addEventListener(Event.SOUND_COMPLETE, soundCompleted);
                }
            }
            return !waitForSoundToFinish || soundComplete;
        }

        private function soundCompleted(e:Event):void
        {
            channel.removeEventListener(Event.SOUND_COMPLETE, soundCompleted);
            soundComplete = true;
        }

        /**
         * clear out any state information the step might be saving and
         * go back to like when it was first created
         */
        public override function restart():void
        {
            if ( (null != channel) && (channel.hasEventListener(Event.SOUND_COMPLETE)) )
            {
                channel.removeEventListener(Event.SOUND_COMPLETE, soundCompleted);
            }

            channel = null;
            soundComplete = false;
        }
    }
}
