package stego.ui
{
    import stego.constants.Colors;
    import stego.gameobject.CanvasGameObject;
    import stego.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class ProgressBar extends CanvasGameObject
    {
        public override function set width(value:Number):void
        {
            super.width = value;
            _needsRedraw = true;
        }

        public override function set height(value:Number):void
        {
            super.height = value;
            _needsRedraw = true;
        }

        /** get/set the low end of the range */
        public function get min():Number { return _min; }
        public function set min(value:Number):void { _min = value; clampValue(); _needsRedraw = true; }
        protected var _min:Number = 0;

        /** get/set the high end of the range */
        public function get max():Number { return _max; }
        public function set max(value:Number):void { _max = value; clampValue(); _needsRedraw = true; }
        protected var _max:Number = 0;

        /** get/set the current value */
        public function get value():Number { return _value; }
        public function set value(arg:Number):void { _value = arg; clampValue(); _needsRedraw = true; }
        protected var _value:Number = 0;

        /** get/set the border width */
        public function get borderWidth():Number { return _borderWidth; }
        public function set borderWidth(value:Number):void { _borderWidth = value; _needsRedraw = true; }
        protected var _borderWidth:Number = 1;

        /** get/set the space between the border and fill. that and height determine fill height */
        public function get padding():Number { return _padding; }
        public function set padding(value:Number):void { _padding = value; _needsRedraw = true; }
        protected var _padding:Number = 1;

        /** get/set the border color */
        public function get borderColor():uint { return _borderColor; }
        public function set borderColor(value:uint):void { _borderColor = value; _needsRedraw = true; }
        protected var _borderColor:uint = Colors.WHITE;

        /** get/set the background color */
        public function get backgroundColor():uint { return _backgroundColor; }
        public function set backgroundColor(value:uint):void { _backgroundColor = value; _needsRedraw = true; }
        protected var _backgroundColor:uint = Colors.BLACK;

        /** get/set the fill color */
        public function get fillColor():uint { return _fillColor; }
        public function set fillColor(value:uint):void { _fillColor = value; _needsRedraw = true; }
        protected var _fillColor:uint = Colors.WHITE;

        /** keep the value between min and max */
        protected function clampValue():void
        {
            _value = (_min > _value) ? _min : ((_max < _value) ? _max: _value);
        }

        /** marks the canvas as dirty */
        protected var _needsRedraw:Boolean = true;

        /** last minute updating before rendering */
        override public function preDraw(deltaTime:Number):void
        {
            super.preDraw(deltaTime);

            if (_needsRedraw)
            {
                _canvas.clear();

                tempRect.x = this.x;
                tempRect.y = this.y;
                tempRect.width = this.width;
                tempRect.height = this.height;
                _canvas.fillRectRgb(tempRect, _borderColor);

                tempRect.inflate(-_borderWidth, -_borderWidth);
                _canvas.fillRectRgb(tempRect, _backgroundColor);

                tempRect.inflate(-_padding, -_padding);
                tempRect.width *= MathUtil.unlerp(_value, _min, _max);
                _canvas.fillRectRgb(tempRect, _fillColor);

                _needsRedraw = false;
            }
        }
    }
}
