﻿package stego.ui
{
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import stego.GameInterface;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingGraphicButton extends PulsingGraphic
    {
        private var mouseEventDummy:Sprite;

        /**
         * Graphic to display for the button when the mouse is not over it
         */
        public override function set graphic(value:Class):void
        {
            _buttonGraphic = value;
            if (!mouseIsOver)
            {
                swapGraphic(_buttonGraphic);
            }
        }

        /**
         * Graphic to display for the button when the mouse is over it
         */
        public function get hoverGraphic():Class { return _hoverGraphic; }
        public function set hoverGraphic(value:Class):void
        {
            _hoverGraphic = value;

            if (mouseIsOver)
            {
                swapGraphic(_hoverGraphic);
            }
        }
        protected var _hoverGraphic:Class;

        /**
         * whether the mouse is currently over the button
         */
        public function get mouseIsOver():Boolean { return _mouseIsOver; }
        public function set mouseIsOver(value:Boolean):void
        {
            _mouseIsOver = value;
            if (value && (null != hoverGraphic) )
            {
                swapGraphic(hoverGraphic);
            }
            if (!value && (null != graphic))
            {
                swapGraphic(graphic);
            }
        }
        protected var _mouseIsOver:Boolean = false;

        /** function to call when the button is clicked */
        public function get callback():Function { return _callback; }
        public function set callback(value:Function):void
        {
            _callback = value;
        }
        private var _callback:Function;

        /**
         * initial setup/preparation for reuse
         */
        public override function init(game:GameInterface):void
        {
            super.init(game);

            if (null == mouseEventDummy)
            {
                mouseEventDummy = new Sprite();
                mouseEventDummy.graphics.beginFill(0, 0);
                mouseEventDummy.graphics.drawRect(0, 0, 1, 1);
                mouseEventDummy.graphics.endFill();
                this.addChild(mouseEventDummy);
                addMouseEventListeners();
            }

            mouseIsOver = false;
        }

        /**
         * cleanup
         */
        public override function shutdown():void
        {
            callback = null;
            removeMouseEventListeners();
            super.shutdown();
        }

        /**
         * change the current graphic to graphicClass
         */
        protected override function swapGraphic(graphicClass:Class):void
        {
            super.swapGraphic(graphicClass);
            removeChild(mouseEventDummy);
            addChild(mouseEventDummy);
            updateMouseEventDummy();
        }

        /**
         * make the dummy mouse event rect match the main graphic
         */
        private function updateMouseEventDummy():void
        {
            var bounds:Rectangle = mainGraphic.getBounds(mouseEventDummy.parent);
            mouseEventDummy.x = bounds.x;
            mouseEventDummy.y = bounds.y;
            mouseEventDummy.width = bounds.width;
            mouseEventDummy.height = bounds.height;
        }

        /** call the callback when we get clicked */
        private function handleClick(e:MouseEvent):void
        {
            if (null != _callback)
            {
                _callback();
            }
        }

        private function handleRollOver(evt:MouseEvent = null):void
        {
            mouseIsOver = true;
        }

        private function handleRollOut(evt:MouseEvent = null):void
        {
            mouseIsOver = false;
        }

        private function addMouseEventListeners():void
        {
            mouseEventDummy.addEventListener(MouseEvent.ROLL_OVER, handleRollOver);
            mouseEventDummy.addEventListener(MouseEvent.ROLL_OUT, handleRollOut);
            mouseEventDummy.addEventListener(MouseEvent.CLICK, handleClick);
        }

        private function removeMouseEventListeners():void
        {
            mouseEventDummy.removeEventListener(MouseEvent.ROLL_OVER, handleRollOver);
            mouseEventDummy.removeEventListener(MouseEvent.ROLL_OUT, handleRollOut);
            mouseEventDummy.removeEventListener(MouseEvent.CLICK, handleClick);
        }
    }

}
