﻿package stego.ui
{
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import stego.constants.Colors;
    import stego.GameInterface;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingTextButton extends PulsingText
    {
        private var _hoverColor:Number = Colors.YELLOW;
        private var _mouseEventDummy:Sprite;

        /** function to call when the button is clicked */
        public function get callback():Function { return _callback; }
        public function set callback(value:Function):void
        {
            _callback = value;
        }
        private var _callback:Function;

        public override function init(game:GameInterface):void
        {
            super.init(game);

            addMouseEventListeners();
        }

        protected override function createGraphics():void
        {
            super.createGraphics();

            if (null == _mouseEventDummy)
            {
                _mouseEventDummy = new Sprite();
                _mouseEventDummy.graphics.beginFill(0, 0);
                _mouseEventDummy.graphics.drawRect(0, 0, 1, 1);
                _mouseEventDummy.graphics.endFill();
                this.addChild(_mouseEventDummy);
            }

            matchDummyToText();
        }

        public function setHoverColor(color:Number):void
        {
            _hoverColor = color;
        }

        public override function shutdown():void
        {
            callback = null;
            removeMouseEventListeners();
            super.shutdown();
        }

        private function addMouseEventListeners():void
        {
            _mouseEventDummy.addEventListener(MouseEvent.ROLL_OVER, handleRollOver);
            _mouseEventDummy.addEventListener(MouseEvent.ROLL_OUT, handleRollOut);
            _mouseEventDummy.addEventListener(MouseEvent.CLICK, handleClick);

            this.addEventListener(MouseEvent.CLICK, fixFocus);
        }

        private function removeMouseEventListeners():void
        {
            _mouseEventDummy.removeEventListener(MouseEvent.ROLL_OVER, handleRollOver);
            _mouseEventDummy.removeEventListener(MouseEvent.ROLL_OUT, handleRollOut);
            _mouseEventDummy.removeEventListener(MouseEvent.CLICK, handleClick);

            this.removeEventListener(MouseEvent.CLICK, fixFocus);
        }

        /** call the callback when we get clicked */
        private function handleClick(e:MouseEvent):void
        {
            if (null != _callback)
            {
                _callback();
            }
        }

        private function handleRollOver(evt:MouseEvent = null):void
        {
            textField.fontColor = _hoverColor;
            pulseTextField.fontColor = _hoverColor;
        }

        private function handleRollOut(evt:MouseEvent = null):void
        {
            textField.fontColor = color;
            pulseTextField.fontColor = color;
        }

        /**
         * workaround for weird flash issue with inputs after clicking a textfield
         */
        private function fixFocus(e:Event = null):void
        {
            // working around flash issue with clicked textfield making the keyboard not work.
            stage.focus = null;
        }

        /**
         * make the dummy mouse event rect match the displayed text
         */
        private function matchDummyToText():void
        {
            var bounds:Rectangle = textField.getBounds(_mouseEventDummy.parent);
            _mouseEventDummy.x = bounds.x;
            _mouseEventDummy.y = bounds.y;
            _mouseEventDummy.width = bounds.width;
            _mouseEventDummy.height = bounds.height;
        }

        public override function centerOn(x:Number, y:Number):void
        {
            super.centerOn(x, y);
            matchDummyToText();
        }
        public override function setFont(font:String):void
        {
            super.setFont(font);
            matchDummyToText();
        }
        public override function setFontSize(desiredSize:Number):void
        {
            super.setFontSize(desiredSize);
            matchDummyToText();
        }
        public override function setText(text:String):void
        {
            super.setText(text);
            matchDummyToText();
        }
    }

}
