package stego.ui
{
    import flash.events.MouseEvent;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import stego.constants.Colors;
    import stego.GameInterface;
	import stego.GameObject;
    import stego.gameobject.CanvasGameObject;
    import stego.util.MathUtil;
	
	/**
     * ...
     * @author Mike Meyer
     */
    public class Slider extends CanvasGameObject
    {
        public override function set width(value:Number):void
        {
            super.width = value;
            _needsRedraw = true;
        }
        public override function set height(value:Number):void
        {
            super.height = value;
            _needsRedraw = true;
        }

        /** get/set the low end of the range */
        public function get min():Number { return _min; }
        public function set min(value:Number):void { _min = value; clampValue(); _needsRedraw = true; }
        protected var _min:Number = 0;

        /** get/set the high end of the range */
        public function get max():Number { return _max; }
        public function set max(value:Number):void { _max = value; clampValue(); _needsRedraw = true; }
        protected var _max:Number = 0;

        /** get/set the current value */
        public function get value():Number { return _value; }
        public function set value(arg:Number):void { _value = arg; clampValue(); _needsRedraw = true; }
        protected var _value:Number = 0;

        /** get/set the border width */
        public function get borderWidth():Number { return _borderWidth; }
        public function set borderWidth(value:Number):void { _borderWidth = value; _needsRedraw = true; }
        protected var _borderWidth:Number = 1;

        /** get/set the space between the border and thumb. that and height determine thumb height */
        public function get padding():Number { return _padding; }
        public function set padding(value:Number):void { _padding = value; _needsRedraw = true; }
        protected var _padding:Number = 1;

        /** get/set the width of the thumb */
        public function get thumbWidth():Number { return _thumbWidth; }
        public function set thumbWidth(value:Number):void { _thumbWidth = value; _needsRedraw = true; }
        protected var _thumbWidth:Number = 20;

        /** get/set the border color */
        public function get borderColor():uint { return _borderColor; }
        public function set borderColor(value:uint):void { _borderColor = value; _needsRedraw = true; }
        protected var _borderColor:uint = Colors.WHITE;

        /** get/set the background color */
        public function get backgroundColor():uint { return _backgroundColor; }
        public function set backgroundColor(value:uint):void { _backgroundColor = value; _needsRedraw = true; }
        protected var _backgroundColor:uint = Colors.BLACK;

        /** get/set the thumb's color */
        public function get thumbColor():uint { return _thumbColor; }
        public function set thumbColor(value:uint):void { _thumbColor = value; _needsRedraw = true; }
        protected var _thumbColor:uint = Colors.WHITE;

        /** keep the value between min and max */
        protected function clampValue():void
        {
            _value = (_min > _value) ? _min : ((_max < _value) ? _max: _value);
        }

        /** marks the canvas as dirty */
        protected var _needsRedraw:Boolean = true;

        /** where the mouse drag started */
        protected var dragStart:Number = 0;

        /** where the thumb was when the mouse drag started */
        protected var dragThumbStart:Number = 0;

        /** whether a drag is happening RIGHT NOW */
        protected var dragging:Boolean = false;

        /** keep track of where the thumb is at */
        protected var thumbRect:Rectangle = new Rectangle();

        /** keep track of the are the thumb is allowed to slide in */
        protected var thumbSlideArea:Rectangle = new Rectangle();

        /**
         * callback for when things change
         * should look like:
         *   funciton sliderChanged(value:Number):void
         */
        public var onChangeCallback:Function;

        /**
         * initial setup/preparation for reuse
         */
        public override function init(game:GameInterface):void
        {
            super.init(game);
            addMouseEventListeners();
        }

        /**
         * cleanup
         */
        public override function shutdown():void
        {
            removeMouseEventListeners();
            onChangeCallback = null;
            super.shutdown();
        }

        /** last minute updating before rendering */
        override public function preDraw(deltaTime:Number):void
        {
            super.preDraw(deltaTime);

            if (_needsRedraw)
            {
                _canvas.fillRgb(_borderColor);

                tempRect.x = _borderWidth;
                tempRect.y = _borderWidth;
                tempRect.width = width - 2 * _borderWidth;
                tempRect.height = height - 2 * _borderWidth;
                _canvas.fillRectRgb(tempRect, _backgroundColor);

                // available area for the thumb
                thumbSlideArea.x = tempRect.x + _padding;
                thumbSlideArea.y = tempRect.y + _padding;
                thumbSlideArea.width = tempRect.width - 2 * _padding;
                thumbSlideArea.height = tempRect.height - 2 * _padding;

                // actual thumb position
                thumbRect.x = MathUtil.remap(_value, _min, _max, thumbSlideArea.x, thumbSlideArea.right - _thumbWidth);
                thumbRect.width = _thumbWidth;
                thumbRect.y = thumbSlideArea.y;
                thumbRect.height = thumbSlideArea.height;

                _canvas.fillRectRgb(thumbRect, _thumbColor);

                _needsRedraw = false;
            }
        }

        private function addMouseEventListeners():void
        {
            addEventListener(MouseEvent.MOUSE_DOWN, handleMouseDown);
            stage.addEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
        }

        private function removeMouseEventListeners():void
        {
            removeEventListener(MouseEvent.MOUSE_DOWN, handleMouseDown);
            stage.removeEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
        }

        private function handleMouseDown(e:MouseEvent):void
        {
            if (thumbRect.contains(e.localX, e.localY))
            {
                dragging = true;
                dragStart = e.localX;
                dragThumbStart = thumbRect.x;
            }
        }

        private function handleMouseUp(e:MouseEvent):void
        {
            dragging = false;
        }

        override public function update(deltaTime:Number):void
        {
            super.update(deltaTime);

            if (dragging)
            {
                var newThumbPosition:Number = (mouseX - dragStart) + dragThumbStart;
                value = MathUtil.remap(newThumbPosition, thumbSlideArea.left, thumbSlideArea.right - thumbRect.width, _min, _max);
                clampValue();

                if (null != onChangeCallback)
                {
                    onChangeCallback(value);
                }
            }
        }
    }

}
