package stego.ui.bitmapfont
{
    import flash.display.BitmapData;
    import flash.utils.Dictionary;
	/**
     * struct for holding info about a bitmap font
     * @author Mike Meyer
     */
    public class BitmapFont
    {
        /** bitmapdata source image */
        public var source:BitmapData;

        /** width of each characters's cell in the source image */
        public var charWidth:Number;

        /** height of each characters's cell in the source image */
        public var charHeight:Number;

        /** optional extra horizontal space between characters */
        public var tracking:Number;

        /** optional extra vertical space between lines */
        public var leading:Number;

        /**
         * override charWidth for specific characters
         * example: bitmapFont.perCharacterWidth["I"] = 4;
         */
        public var perCharacterWidth:Dictionary;

        /**
         * the characters that the BitmapFont contains, in order
         */
        public var characters:String;

        /** todo: kerning pairs? */

        /**
         * constructor
         * @param	source      bitmapdata from which character images can be pulled
         * @param	charWidth   width of each character image in the image
         * @param	charHeight  height of each character image in the image
         * @param	tracking    optional extra horizontal space between characters
         * @param	leading     optional extra vertical space between lines
         */
        public function BitmapFont( source:BitmapData,
                                    charWidth:Number,
                                    charHeight:Number,
                                    characters:String,
                                    tracking:Number = 0,
                                    leading:Number = 0 )
        {
            this.source = source;
            this.charWidth = charWidth;
            this.charHeight = charHeight;
            this.characters = characters;
            this.tracking = tracking;
            this.leading = leading;
            perCharacterWidth = new Dictionary();

            if ((source.width / charWidth) != Math.floor(source.width / charWidth))
            {
                throw new Error("bitmap font image width is not a multiple of the character width!!!");
            }
        }
    }
}
