package stego.ui.bitmapfont
{
    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import flash.display.PixelSnapping;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import flash.utils.Dictionary;
    import stego.GameInterface;
	import stego.GameObject;
    import stego.util.graphics.Canvas;
    import stego.util.MathUtil;
	
	/**
     * @author Mike Meyer
     */
    public class BitmapFontTextField extends GameObject
    {
        [Embed(source='../../assets/Fonts/stegobitmapfontbordered.png')]
        protected static const _defaultBitmapFont:Class;
        protected static const defaultBitmapFontImage:BitmapData = (new _defaultBitmapFont).bitmapData;

        /** set up the default stego bitmap font */
        protected static function createDefaultBitmapFont():BitmapFont
        {
            var bitmapFont:BitmapFont = new BitmapFont( defaultBitmapFontImage,
                                                        8,
                                                        11,
                                                        " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~",
                                                        0,
                                                        0 );
            bitmapFont.perCharacterWidth["!"] = 6;
            bitmapFont.perCharacterWidth["\""] = 7;
            bitmapFont.perCharacterWidth["#"] = 7;
            bitmapFont.perCharacterWidth["$"] = 7;
            bitmapFont.perCharacterWidth["'"] = 4;
            bitmapFont.perCharacterWidth["("] = 5;
            bitmapFont.perCharacterWidth[")"] = 5;
            bitmapFont.perCharacterWidth["*"] = 7;
            bitmapFont.perCharacterWidth["+"] = 7;
            bitmapFont.perCharacterWidth[","] = 5;
            bitmapFont.perCharacterWidth["-"] = 5;
            bitmapFont.perCharacterWidth["."] = 4;
            bitmapFont.perCharacterWidth[":"] = 4;
            bitmapFont.perCharacterWidth[";"] = 4;
            bitmapFont.perCharacterWidth["<"] = 6;
            bitmapFont.perCharacterWidth[">"] = 6;
            bitmapFont.perCharacterWidth["I"] = 4;
            bitmapFont.perCharacterWidth["["] = 5;
            bitmapFont.perCharacterWidth["]"] = 5;
            bitmapFont.perCharacterWidth["_"] = 7;
            bitmapFont.perCharacterWidth["`"] = 7;
            bitmapFont.perCharacterWidth["i"] = 4;
            bitmapFont.perCharacterWidth["{"] = 5;
            bitmapFont.perCharacterWidth["|"] = 4;
            bitmapFont.perCharacterWidth["}"] = 5;
            return bitmapFont;
        }

        /** default bitmap font */
        public static const defaultBitmapFont:BitmapFont = createDefaultBitmapFont();

        /** avoid new */
        protected static var _tempRect:Rectangle = new Rectangle();

        /** avoid new */
        protected static var _tempPoint:Point = new Point();

        /**
         * constant for horizontal alignment
         * set hAlign to this and the left edge of the text
         * will be at this object's x position
         */
        public static const HALIGN_LEFT:uint       = 0;

        /**
         * constant for horizontal alignment
         * set hAlign to this and the center of the text
         * will be at this object's x position
         */
        public static const HALIGN_CENTER:uint     = 1;

        /**
         * constant for horizontal alignment
         * set hAlign to this and the right edge of the text
         * will be at this object's x position
         */
        public static const HALIGN_RIGHT:uint      = 2;


        /**
         * constant for vertical alignment
         * set hAlign to this and the top edge of the text
         * will be at this object's y position
         */
        public static const VALIGN_TOP:uint        = 3;


        /**
         * constant for vertical alignment
         * set hAlign to this and the center of the text
         * will be at this object's y position
         */
        public static const VALIGN_MIDDLE:uint     = 4;


        /**
         * constant for vertical alignment
         * set hAlign to this and the bottom edge of the text
         * will be at this object's y position
         */
        public static const VALIGN_BOTTOM:uint     = 5;

        /**
         * horizontal alignement.
         * should be one of the values from BitmapFontTextField.HALIGN_*
         */
        public function get hAlign():uint { return _hAlign; }
        public function set hAlign(value:uint):void
        {
            if (_hAlign != value )
            {
                _hAlign = value;
                _needsRedraw = true;
            }
        }
        private var _hAlign:uint = HALIGN_LEFT;

        /**
         * vertical alignement.
         * should be one of the values from BitmapFontTextField.VALIGN_*
         */
        public function get vAlign():uint { return _vAlign; }
        public function set vAlign(value:uint):void
        {
            _vAlign = value;
            updateVAlign();
        }
        private var _vAlign:uint = VALIGN_TOP;

        /** bitmapfont this renders with */
        public function get bitmapFont():BitmapFont { return _bitmapFont; }
        public function set bitmapFont(value:BitmapFont):void
        {
            if (value != _bitmapFont)
            {
                _bitmapFont = value;
                _needsRedraw = true;
                _sizingUpToDate = false;
            }
        }
        protected var _bitmapFont:BitmapFont = defaultBitmapFont;

        /** whether the text needs to be redrawn */
        protected var _needsRedraw:Boolean = false;

        /** the text that this is to display */
        public function get text():String { return _text; }
        public function set text(value:String):void
        {
            if (value != _text)
            {
                _text = value;
                _needsRedraw = true;
                _sizingUpToDate = false;
            }
        }
        protected var _text:String = "";

        /** scaling applied to the rendered text */
        public function get fontScale():Number { return _fontScale; }
        public function set fontScale(value:Number):void
        {
            if (value != _fontScale)
            {
                _bitmap.scaleX = value;
                _bitmap.scaleY = value;
                _fontScale = value;
                updateHAlign();
                updateVAlign();
            }
        }
        protected var _fontScale:Number = 1;

        /**
         * optional extra horizontal space between characters
         * (added to that of the font!)
         */
        public function get tracking():Number { return _tracking; }
        public function set tracking(value:Number):void
        {
            if (value != _tracking)
            {
                _tracking = value;
                _needsRedraw = true;
                _sizingUpToDate = false;
            }
        }
        protected var _tracking:Number = 0;

        /**
         * optional extra vertical space between lines
         * (added to that of the font!)
         */
        public function get leading():Number { return _leading; }
        public function set leading(value:Number):void
        {
            if (value != _leading)
            {
                _leading = value;
                _needsRedraw = true;
                _sizingUpToDate = false;
            }
        }
        protected var _leading:Number = 0;

        /** canvas we draw onto */
        public function get canvas():Canvas { return _canvas; }
        protected var _canvas:Canvas;

        /** the bitmap used to render it */
        protected var _bitmap:Bitmap;

        /** width of the rendered text */
        public function get textWidth():Number
        {
            if (!_sizingUpToDate)
            {
                reCalcSize();
            }
            return _prescaleTextWidth * _fontScale;
        }
        protected var _prescaleTextWidth:Number = 0;

        /** width of the rendered text */
        public function get textHeight():Number
        {
            if (!_sizingUpToDate)
            {
                reCalcSize();
            }
            return _prescaleTextHeight * _fontScale;
        }
        protected var _prescaleTextHeight:Number = 0;

        /** line widths. like _textWidth, it needs to be updated (you can check the _sizingUpToDate flag) */
        protected var _lineWidths:Array;

        /** whether _textWidth and _textHeight need updating */
        protected var _sizingUpToDate:Boolean = true;

        /**
         * initial setup
         */
        override public function init(game:GameInterface):void
        {
            super.init(game);
            resetToDefaults();

            _lineWidths = [];

            if (!_bitmap)
            {
                _bitmap = new Bitmap(null, PixelSnapping.NEVER);
                addChild(_bitmap);
            }

            if (!_canvas)
            {
                _canvas = new Canvas(_bitmap);
            }
            else
            {
                _canvas.target = _bitmap;
                _canvas.clear();
            }
        }

        /**
         * cleanup
         */
        override public function shutdown():void
        {
            super.shutdown();
            if (_canvas)
            {
                _canvas.destroy();
                _canvas = null;
            }
            if (_bitmap)
            {
                removeChild(_bitmap);
                _bitmap = null;
            }
        }

        /**
         * go to the default state
         */
        protected function resetToDefaults():void
        {
            _hAlign = HALIGN_LEFT;
            _vAlign = VALIGN_TOP;
            _bitmapFont = defaultBitmapFont;
            _text = "";
            _fontScale = 1;
            _tracking = 0;
            _leading = 0;
            if (_canvas)
            {
                _canvas.clear();
            }
            _sizingUpToDate = false;
        }

        /**
         * last chance to draw to _bitmap before it gets rendered
         */
        override public function preDraw(deltaTime:Number):void
        {
            super.preDraw(deltaTime);
            if (_needsRedraw)
            {
                drawText();
                _needsRedraw = false;
            }
        }

        /**
         * render the text to the canvas
         */
        protected function drawText():void
        {
            adjustCanvasSize();
            _canvas.clear();
            renderTextToCanvas();
        }

        /** make sure the canvas is big enough to hold the text */
        protected function adjustCanvasSize():void
        {
            if (!_sizingUpToDate)
            {
                reCalcSize();
            }
            if (_canvas.width < _prescaleTextWidth) { _canvas.width = _prescaleTextWidth; }
            if (_canvas.height < _prescaleTextHeight) { _canvas.height = _prescaleTextHeight; }
        }

        /**
         * loop over the string and update _textWidth and _textHeight to
         * reflect new settings
         */
        protected function reCalcSize():void
        {
            _prescaleTextWidth = 0;
            _prescaleTextHeight = 0;

            if (null != _text)
            {
                var len:int = _text.length,
                    i:int = 0,
                    lineNumber:int = 0,
                    characters:String = _bitmapFont.characters,
                    globalCharWidth:Number = _bitmapFont.charWidth,
                    globalCharHeight:Number = _bitmapFont.charHeight,
                    tracking:Number = _bitmapFont.tracking + _tracking,
                    leading:Number = _bitmapFont.leading + _leading,
                    charWidthOverrides:Dictionary = _bitmapFont.perCharacterWidth,
                    charWidth:Number = _bitmapFont.charWidth, // used inside the loop as the possibly-overriden curren-character's width
                    lineWidth:Number = 0, // width of the current line so far
                    char:String,          // character found in the string we are drawing
                    prevChar:String = "", // character found in the string we are drawing last time through the loop
                    charIndex:int;     // index into characters
                _lineWidths.length = 0;
                for (; i < len; ++i)
                {
                    // newline! (or first line)
                    if ( (prevChar == "\n") ||
                         (i==0) )
                    {
                        _prescaleTextHeight += globalCharHeight;
                        // there's also leading, but not for the first line
                        if (i != 0) { _prescaleTextHeight += leading;  }
                        lineWidth = 0;
                    }

                    char = _text.charAt(i);
                    charIndex = characters.indexOf(char);

                    // renderable?
                    if (charIndex != -1)
                    {
                        charWidth = (charWidthOverrides.hasOwnProperty(char)) ? charWidthOverrides[char] : globalCharWidth;
                        // todo: check for word wrapping
                        lineWidth += (charWidth + tracking);
                        _prescaleTextWidth = Math.max(_prescaleTextWidth, lineWidth);
                    }

                    // newline! (or last line)
                    if ( (char == "\n") ||
                         (i + 1 == len) )
                    {
                        _lineWidths[lineNumber] = lineWidth;
                        ++lineNumber;
                    }

                    prevChar = char;
                }
            }
            _sizingUpToDate = true;
        }

        /**
         * loop over the string and start pasting characters
         * out of the source image into the canvas
         */
        protected function renderTextToCanvas():void
        {
            if (null != _text)
            {
                var len:int = _text.length,
                    i:int = 0,
                    lineNumber:int = 0,
                    lineWidth:Number,
                    characters:String = _bitmapFont.characters,
                    globalCharWidth:Number = _bitmapFont.charWidth,
                    globalCharHeight:Number = _bitmapFont.charHeight,
                    sourceImage:BitmapData = _bitmapFont.source,
                    sourceImageWidth:Number = sourceImage.width,
                    sourceImageColumns:int = Math.floor(sourceImageWidth / globalCharWidth),
                    tracking:Number = _bitmapFont.tracking + _tracking,
                    leading:Number = _bitmapFont.leading + _leading,
                    charWidthOverrides:Dictionary = _bitmapFont.perCharacterWidth,
                    charWidth:Number = _bitmapFont.charWidth, // used inside the loop as the possibly-overriden curren-character's width
                    char:String,          // character found in the string we are drawing
                    charIndex:int,        // index into characters
                    x:Number = 0,
                    y:Number = 0;
                for (; i < len; ++i)
                {
                    // newline! (or first line)
                    if ( (char == "\n") ||
                         (i==0) )
                    {
                        lineWidth = _lineWidths[lineNumber];
                        switch (_hAlign)
                        {
                            case HALIGN_LEFT:
                                x = 0;
                                break;
                            case HALIGN_CENTER:
                                x = 0.5 * (_prescaleTextWidth - lineWidth);
                                break;
                            case HALIGN_RIGHT:
                                x = _prescaleTextWidth - lineWidth;
                                break;
                        }
                    }

                    char = _text.charAt(i);
                    charIndex = characters.indexOf(char);

                    // renderable?
                    if (charIndex != -1)
                    {
                        charWidth = (charWidthOverrides.hasOwnProperty(char)) ? charWidthOverrides[char] : globalCharWidth;

                        _tempRect.x = (charIndex % sourceImageColumns) * globalCharWidth;
                        _tempRect.y = Math.floor(charIndex / sourceImageColumns) * globalCharHeight;
                        _tempRect.width = charWidth;
                        _tempRect.height = globalCharHeight;
                        _tempPoint.x = x;
                        _tempPoint.y = y;
                        _canvas.copyPixels(sourceImage, _tempRect, _tempPoint, true);

                        // todo: check for word wrapping
                        x += (charWidth + tracking);
                    }

                    // newline!
                    if (char == "\n")
                    {
                        ++lineNumber;
                        y += (globalCharHeight + leading);
                    }
                }
                updateHAlign();
                updateVAlign();
            }
        }

        /** reposition the text based on the horizontal alignment setting */
        protected function updateHAlign():void
        {
            switch(_hAlign)
            {
                case HALIGN_LEFT:
                    _bitmap.x = 0;
                    break;
                case HALIGN_CENTER:
                    _bitmap.x = -0.5 * textWidth;
                    break;
                case HALIGN_RIGHT:
                    _bitmap.x = -textWidth;
                    break;
            }
        }

        /** reposition the text based on the vertical alignment setting */
        protected function updateVAlign():void
        {
            switch(_vAlign)
            {
                case VALIGN_TOP:
                    _bitmap.y = 0;
                    break;
                case VALIGN_MIDDLE:
                    _bitmap.y = -0.5 * textHeight;
                    break;
                case VALIGN_BOTTOM:
                    _bitmap.y = -textHeight;
                    break;
            }
        }
    }
}
