package stego.util
{
    import flash.events.Event;
    import flash.display.Stage;
    import flash.events.KeyboardEvent;

    /**
     * [broadcast event] Dispatched when Flash Player loses operating system focus and is becoming inactive.
     * @eventType stego.util.Cheat.CHEATENTERED
     */
    [Event(name="cheatentered", type="flash.events.Event")]

    /**
     * Utility class for adding cheat codes.
     * Register the string to type, and associate a callback with it.
     *
     * Example use:
     *
     *      var cheats:CheatUtil = new CheatUtil();
     *      cheats.Init(stage);
     *      cheats.AddCheatCode("next", Music.StartNewMusic);
     *
     * @author Mike Meyer
     */
    public class CheatUtil
    {
        private var stage:Stage;
        private var keysEntered:String = "";
        private static const KEYLOG_MAX_LENGTH:uint = 16;

        /// used as a hash from cheatCodes(strings) to callbacks(arrays of functions)
        private var _cheats:Object;

        /**
         * Get the Cheat utility ready for use
         * @param	stage - stage instance.  used so that this thing can
         *                  listen to keyDown
         */
        public function init(stage:Stage):void
        {
            stage = stage;
            stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);

            _cheats = {};
        }

        /**
         * Clean up the cheat util
         */
        public function shutdown():void
        {
            stage.removeEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
            stage = null;
            _cheats = null;
        }

        /**
         * Add a cheat code to the util
         * @param	code      string to type to make your callback get called
         * @param	callback  callback to make the cheat happen with
         */
        public function addCheatCode(code:String, callback:Function):void
        {
            if (0 < code.length)
            {
                if (_cheats.hasOwnProperty(code))
                {
                    var callbacks:Array = _cheats[code];
                    if (-1 == callbacks.indexOf(callback))
                    {
                        callbacks.push(callback);
                    }
                }
                else
                {
                    _cheats[code] = [callback];
                }
            }
        }

        /**
         * remove a cheat code from the util
         * @param	code      should match the AddCheatCode call you are undoing
         * @param	callback  should match the AddCheatCode call you are undoing
         */
        public function removeCheatCode(code:String, callback:Function):void
        {
            if (_cheats.hasOwnProperty(code))
            {
                var callbacks:Array = _cheats[code];
                var index:int = callbacks.indexOf(callback);
                if ( -1 != index )
                {
                    callbacks.splice(index, 1);
                }
            }
        }

        /**
         * watch for the key presses, and when a cheat code has been typed, fire off the appropriate callbacks
         */
        private function handleKeyDown(e:KeyboardEvent):void
        {
            if (0 != e.charCode)
            {
                keysEntered += String.fromCharCode(e.charCode);

                if (keysEntered.length > KEYLOG_MAX_LENGTH)
                {
                    keysEntered = keysEntered.substring(1);
                }

                for (var cheat:String in _cheats)
                {
                    if ( StringUtil.endsWith( keysEntered, cheat ) )
                    {
                        keysEntered = "";
                        var callbacks:Array = _cheats[cheat];
                        for each (var callback:Function in callbacks)
                        {
                            callback();
                        }
                    }
                }
            }
        }
    }
}
