﻿package stego.util
{
    import flash.display.Stage;
    import flash.events.KeyboardEvent;
    import flash.events.MouseEvent;
    import flash.geom.Point;
    import flash.ui.Keyboard;

    /**
     * ...
     * @author Mike Meyer
     */
    public class InputUtil
    {
        public static const KEYSTATE_UP:uint           = 0;
        public static const KEYSTATE_JUSTPRESSED:uint  = 1;
        public static const KEYSTATE_HELD:uint         = 2;
        public static const KEYSTATE_JUSTRELEASED:uint = 3;

        /** sparse array of keystates indexed by keyCode */
        private var keyStates:Object = new Object();
        private var mouseButtonState:Number = InputUtil.KEYSTATE_UP;
        private var _mousePosition:Point         = new Point();
        private var previoumousePosition:Point = new Point();
        private var stage:Stage;

        /**
         * Get InputUtil ready to be used
         */
        public function InputUtil(stage:Stage)
        {
            this.stage = stage;
            this.stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
            this.stage.addEventListener(KeyboardEvent.KEY_UP,   keyUpHandler);
            this.stage.addEventListener(MouseEvent.MOUSE_DOWN,  mouseDownHandler);
            this.stage.addEventListener(MouseEvent.MOUSE_UP,    mouseUpHandler);
            this.stage.addEventListener(MouseEvent.ROLL_OUT,    mouseUpHandler);
        }

        /**
         * clean up
         */
        public function shutdown():void
        {
            stage.removeEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
            stage.removeEventListener(KeyboardEvent.KEY_UP,   keyUpHandler);
            stage.removeEventListener(MouseEvent.MOUSE_DOWN,  mouseDownHandler);
            stage.removeEventListener(MouseEvent.MOUSE_UP,    mouseUpHandler);
            stage.removeEventListener(MouseEvent.ROLL_OUT,    mouseUpHandler);
        }

        /**
         * make the JUST X'ed states go to the more permanent ones
         */
        public function update():void
        {
            for (var key:String in keyStates)
            {
                switch (keyStates[key])
                {
                    case InputUtil.KEYSTATE_JUSTPRESSED:
                        keyStates[key] = InputUtil.KEYSTATE_HELD;
                        break;
                    case InputUtil.KEYSTATE_JUSTRELEASED:
                        keyStates[key] = InputUtil.KEYSTATE_UP;
                        break;
                }
            }

            // mouse button state
            switch (mouseButtonState)
            {
                case InputUtil.KEYSTATE_JUSTPRESSED:
                    mouseButtonState = InputUtil.KEYSTATE_HELD;
                    break;
                case InputUtil.KEYSTATE_JUSTRELEASED:
                    mouseButtonState = InputUtil.KEYSTATE_UP;
                    break;
            }

            // mouse position
            previoumousePosition.x = _mousePosition.x;
            previoumousePosition.y = _mousePosition.y;
            _mousePosition.x = stage.mouseX;
            _mousePosition.y = stage.mouseY;
        }

        /**
         * update the states when a key is pressed
         */
        private function keyDownHandler(e:KeyboardEvent):void
        {
            if (keyStates.hasOwnProperty(e.keyCode))
            {
                switch (keyStates[e.keyCode])
                {
                    case InputUtil.KEYSTATE_UP:
                    case InputUtil.KEYSTATE_JUSTRELEASED:
                        keyStates[e.keyCode] = InputUtil.KEYSTATE_JUSTPRESSED;
                        break;
                    case InputUtil.KEYSTATE_JUSTPRESSED:
                    case InputUtil.KEYSTATE_HELD:
                        // stay there, let Update handle it
                        break;
                }
            }
            else
            {
                keyStates[e.keyCode] = InputUtil.KEYSTATE_JUSTPRESSED;
            }
        }

        /**
         * update the states when a key is released
         */
        private function keyUpHandler(e:KeyboardEvent):void
        {
            if (keyStates.hasOwnProperty(e.keyCode))
            {
                switch (keyStates[e.keyCode])
                {
                    case InputUtil.KEYSTATE_UP:
                    case InputUtil.KEYSTATE_JUSTRELEASED:
                        // stay there
                        break;
                    case InputUtil.KEYSTATE_JUSTPRESSED:
                    case InputUtil.KEYSTATE_HELD:
                        keyStates[e.keyCode] = InputUtil.KEYSTATE_JUSTRELEASED;
                        break;
                }
            }
            else
            {
                keyStates[e.keyCode] = InputUtil.KEYSTATE_JUSTRELEASED;
            }
        }

        /**
         * tell whether any of the keys in the array are down
         */
        public function anyDown(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (isDown(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array are up
         */
        public function anyUp(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (isUp(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array have been up for more than a frame
         */
        public function anyReleased(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (isReleased(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array have been down for more than a frame
         */
        public function anyHeld(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (isHeld(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array were just pressed
         */
        public function anyJustPressed(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (wasJustPressed(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array were just released
         */
        public function anyJustReleased(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (wasJustReleased(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether a key is down or not
         */
        public function isDown(keyCode:uint):Boolean
        {
            if (keyStates.hasOwnProperty(keyCode))
            {
                return ( (keyStates[keyCode] == InputUtil.KEYSTATE_HELD) || (keyStates[keyCode] == InputUtil.KEYSTATE_JUSTPRESSED) );
            }
            else
            {
                return false;
            }
        }

        /**
         * tell whether a key is up or not
         */
        public function isUp(keyCode:uint):Boolean
        {
            if (keyStates.hasOwnProperty(keyCode))
            {
                return ( (keyStates[keyCode] == InputUtil.KEYSTATE_UP) || (keyStates[keyCode] == InputUtil.KEYSTATE_JUSTRELEASED) );
            }
            else
            {
                return true;
            }
        }

        /**
         * tell whether a key has been released for a frame or more
         */
        public function isReleased(keyCode:uint):Boolean
        {
            if (keyStates.hasOwnProperty(keyCode))
            {
                return (keyStates[keyCode] == InputUtil.KEYSTATE_UP);
            }
            else
            {
                return true;
            }
        }

        /**
         * tell whether a key has been held for a frame or more
         */
        public function isHeld(keyCode:uint):Boolean
        {
            if (keyStates.hasOwnProperty(keyCode))
            {
                return (keyStates[keyCode] == InputUtil.KEYSTATE_HELD);
            }
            else
            {
                return false;
            }
        }

        /**
         * tell whether a key was just pressed this frame
         */
        public function wasJustPressed(keyCode:uint):Boolean
        {
            if (keyStates.hasOwnProperty(keyCode))
            {
                return (keyStates[keyCode] == InputUtil.KEYSTATE_JUSTPRESSED);
            }
            else
            {
                return false;
            }
        }

        /**
         * tell whether a key was just released this frame
         */
        public function wasJustReleased(keyCode:uint):Boolean
        {
            if (keyStates.hasOwnProperty(keyCode))
            {
                return (keyStates[keyCode] == InputUtil.KEYSTATE_JUSTRELEASED);
            }
            else
            {
                return false;
            }
        }

        /**
         * get the keystate directly
         */
        public function getKeyState(keyCode:uint):uint
        {
            if (keyStates.hasOwnProperty(keyCode))
            {
                return keyStates[keyCode];
            }
            else
            {
                return InputUtil.KEYSTATE_UP;
            }
        }

        /**
         * deal with mouse down events -- start mouse-looking
         */
        private function mouseDownHandler(e:MouseEvent):void
        {
            switch (mouseButtonState)
            {
                case InputUtil.KEYSTATE_UP:
                case InputUtil.KEYSTATE_JUSTRELEASED:
                    mouseButtonState = InputUtil.KEYSTATE_JUSTPRESSED;
                    break;
                case InputUtil.KEYSTATE_JUSTPRESSED:
                case InputUtil.KEYSTATE_HELD:
                    // stay there, let Update handle it
                    break;
            }
        }

        /**
         * deal with mouse down events -- stop mouse-looking
         */
        private function mouseUpHandler(e:MouseEvent):void
        {
            switch (mouseButtonState)
            {
                case InputUtil.KEYSTATE_UP:
                case InputUtil.KEYSTATE_JUSTRELEASED:
                    // stay there
                    break;
                case InputUtil.KEYSTATE_JUSTPRESSED:
                case InputUtil.KEYSTATE_HELD:
                    mouseButtonState = InputUtil.KEYSTATE_JUSTRELEASED;
                    break;
            }
        }

        /**
         * tell whether the mouse button is down or not
         */
        public function get mouseButtonDown():Boolean
        {
            return ( (mouseButtonState == InputUtil.KEYSTATE_HELD) || (mouseButtonState== InputUtil.KEYSTATE_JUSTPRESSED) );
        }

        /**
         * tell whether the mouse button is up or not
         */
        public function get mouseButtonUp():Boolean
        {
            return ( (mouseButtonState == InputUtil.KEYSTATE_UP) || (mouseButtonState== InputUtil.KEYSTATE_JUSTRELEASED) );
        }

        /**
         * tell whether the mouse button has been held more than a frame
         */
        public function get mouseButtonHeld():Boolean
        {
            return (mouseButtonState == InputUtil.KEYSTATE_HELD);
        }

        /**
         * tell whether the mouse button button was just released this frame
         */
        public function get mouseButtonJustReleased():Boolean
        {
            return (mouseButtonState == InputUtil.KEYSTATE_JUSTRELEASED);
        }

        /**
         * tell whether the mouse button button was just pressed this frame
         */
        public function get mouseButtonJustPressed():Boolean
        {
            return (mouseButtonState == InputUtil.KEYSTATE_JUSTPRESSED);
        }

        /**
         * get the mouse position
         */
        public function get mousePosition():Point
        {
            return _mousePosition.clone();
        }

        /**
         * get how much the mouse since the previous frame
         */
        public function get mouseMovement():Point
        {
            return _mousePosition.subtract(previoumousePosition);
        }
    }
}
