﻿package stego.util.music
{
    import flash.events.EventDispatcher;
    import flash.events.IOErrorEvent;
    import stego.util.ArrayUtil;
    import stego.util.music.MusicEvent;
    import stego.util.music.Song;

    /**
     * Class to handle randomizing, looping, and streaming a playlist
     *
     * Example use:
     *
     *      var music:MusicUtil - new Music;
     *      music.Init( [ new StreamedSong( "http://8bitcollective.com/items/music/joule-girl_scrotum.mp3",
     *                                      "Girl Scrotum",
     *                                      "joule",
     *                                      "http://8bitcollective.com/music/joule/girl+scrotum+%28OHC%29/"),
     *                    new StreamedSong( "http://8bitcollective.com/items/music/Untill%20I%20Sleep.mp3",
     *                                      "Until I Sleep",
     *                                      "henryhomesweet",
     *                                      "http://8bitcollective.com/music/henryhomesweet/Until+I+Sleep/") ] );
     *      music.StartNewMusic();
     *
     * @author Mike Meyer
     */
    public class MusicUtil extends EventDispatcher
    {
        /**
         * currently playing song.  may be null
         */
        public function get currentSong():Song { return _currentSong; }
        private var _currentSong:Song;

        public function get isPlaying():Boolean { return ((null!=_currentSong)&&_currentSong.isPlaying); }

        private var _songList:Array;
        private var internalSongList:Array; // song list from which we have cull any broken links
        /**
         * List of songs that MusicUtil will play from
         */
        public function get songList():Array { return _songList; }

        public function get isPaused():Boolean { return (null!=_currentSong)&&_currentSong.isPaused; }

        /**
         * List of songs that MusicUtil will play from
         */
        public function set songList(value:Array):void
        {
            if (null != value)
            {
                _songList = value.concat();
                internalSongList = value.concat();
            }
            else
            {
                _songList = [];
                internalSongList = [];
            }
        }

        /**
         * Get the MusicUtil ready for use.  Note that if you don't provide
         * any songs here, you better do a .SongList = [...];
         * @param	playlist and array of stego.util.music.Songs
         */
        public function init(playlist:/*stego.util.music.Song*/Array = null):void
        {
            songList = playlist;
        }
        /**
         * Clean up anything this could possibly leave hanging
         */
        public function shutdown():void
        {
            stopMusic();
            cleanUpSong();
        }

        /**
         * The song is going away or turned out to be invalid or something.
         * lose references to it
         */
        private function cleanUpSong():void
        {
            if (null != _currentSong)
            {
                _currentSong.onSongCompleteCb = null;
                _currentSong.onSongBeginCb = null;
                _currentSong.onFailCb = null;
                _currentSong.stop();
                _currentSong = null;
            }
        }

        /**
         * Start looping a random song.
         */
        public function startNewMusic():void
        {
            cleanUpSong();

            if (internalSongList.length > 0)
            {
                _currentSong = ArrayUtil.randomOneOf(internalSongList);
                _currentSong.onFailCb = onFail;
                _currentSong.onSongBeginCb = onSongBegin;
                _currentSong.onSongCompleteCb = startNewMusic;
                _currentSong.start();
            }
        }

        /**
         * music has been successfully found and can start playing
         */
        private function onSongBegin():void
        {
            dispatchEvent(new MusicEvent(MusicEvent.EVENT_NEW_SONG, currentSong));
        }

        /**
         *
         */
        private function onFail(e:IOErrorEvent):void
        {
            ArrayUtil.removeEqual(internalSongList, _currentSong);
            startNewMusic();
        }

        /**
         * Stop the current song
         */
        public function stopMusic():void
        {
            if (null != _currentSong)
            {
                _currentSong.stop();
            }
        }

        /**
         * Pause the current song
         */
        public function pause():void
        {
            if ((null != _currentSong) && !isPaused)
            {
                _currentSong.pause();
            }
        }

        /**
         * Resume after pausing
         */
        public function resume():void
        {
            if (null != _currentSong && isPaused)
            {
                _currentSong.resume();
            }
        }
    }

}
