﻿package stego.util.music
{
    import flash.events.Event;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import flash.media.SoundTransform;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Song
    {
        protected var sound:Sound;
        protected var soundChannel:SoundChannel;
        protected var pausePosition:Number;

        protected var _title:String;
        public function get title():String { return _title; }

        protected var _artist:String;
        public function get artist():String { return _artist; }

        protected var _link:String;
        public function get link():String { return _link; }

        protected var _volume:Number;
        public function get volume():Number { return _volume; }
        public function set volume(value:Number):void
        {
            if (_volume == value) { return; }
            _volume = value;

            if (soundChannel)
            {
                // get to use a temp var and reassign to have the new settings take effect.  Flash is weird like that.
                var st:SoundTransform = soundChannel.soundTransform;
                st.volume = _volume;
                soundChannel.soundTransform = st;
            }
        }

        /** optional callback for what to do if the song fails to play for some reason */
        public var onFailCb:Function;

        /** optional callback for what to do when the song finishes */
        public var onSongCompleteCb:Function;

        /** optional callback for what to do when the song begins (which might not be immediately after start()!) */
        public var onSongBeginCb:Function;

        public function get isPlaying():Boolean { return (null!=soundChannel); }

        public function get isPaused():Boolean { return _isPaused; }
        protected var _isPaused:Boolean;

        public function Song(title:String, artist:String, link:String=null, volume:Number = 0.5)
        {
            _title = title;
            _artist = artist;
            _link = link;
            _volume = volume;
            _isPaused = false;
        }

        public function start():void
        {
            // override me!!!
            _isPaused = false;
        }
        public function stop():void
        {
            // TODO: subclasses may need to override this and queue up stop
            // commands due to async loading.
            if (null != soundChannel)
            {
                soundChannel.removeEventListener(Event.SOUND_COMPLETE, onSongComplete);
                soundChannel.stop();
                soundChannel = null;
            }

            pausePosition = 0;
            _isPaused = false;
        }
        /**
         * Pause the current song
         */
        public function pause():void
        {
            // TODO: subclasses may need to override this and queue up pause
            // commands due to async loading.
            if (null != soundChannel)
            {
                pausePosition = soundChannel.position;
                soundChannel.stop();
                soundChannel = null;
                _isPaused = true;
            }
        }
        /**
         * Resume after pausing
         */
        public function resume():void
        {
            // don't resume from a stop
            if (isPaused)
            {
                beginPlaying(null, pausePosition);
                _isPaused = false;
            }
        }

        /**
         * only good once this.sound has been set up!!!
         * returns false if it fails to actually start playing
         */
        protected function beginPlaying(e:Event = null, position:Number = 0):Boolean
        {
            var itWorked:Boolean = false;
            if (null != sound)
            {
                stop();

                soundChannel = sound.play(position, 1, new SoundTransform(_volume) );
                if (null != soundChannel)
                {
                    itWorked = true;
                    soundChannel.addEventListener(Event.SOUND_COMPLETE, onSongComplete);
                }
            }
            return itWorked;
        }

        /**
         * wrapper around StartNewMusic that can be used as an event listener
         */
        private function onSongComplete(e:Event = null):void
        {
            if (null != onSongCompleteCb)
            {
                onSongCompleteCb();
            }
        }
    }

}
