﻿package stego.util.scene
{
    import flash.display.DisplayObjectContainer;
    import flash.display.Sprite;
    import stego.collections.LinkedList;
    import stego.collections.LinkedListNode;
    import stego.GameInterface;
    import stego.util.GraphicsUtil;
    import stego.util.ObjectPool;

    /**
     * holder for all the scene layers
     * @author Mike Meyer
     */
    public class Scene
    {
        public var layers:/*GameObjectHolder*/LinkedList;
        public var container:DisplayObjectContainer;
        public var game:GameInterface;

        /**
         * remove a single layer from the scene
         */
        public function remove(layer:GameObjectHolder):void
        {
            container.removeChild(layer.container);
            ObjectPool.returnObject(Sprite, layer.container);

            layer.removeAll();
            layer.container = null;
            layer.game = null;
            ObjectPool.returnObject(GameObjectHolder, layer);

            layers.removeByValue(layer);
        }

        /**
         * call RemoveAll on each layer
         */
        public function removeAll():void
        {
            for (var node:LinkedListNode = layers.first; node!=null; node=node.next)
            {
                var layer:GameObjectHolder = node.value;

                layer.removeAll();

                container.removeChild(layer.container);
                ObjectPool.returnObject(Sprite, layer.container);

                layer.container = null;
                layer.game = null;
                ObjectPool.returnObject(GameObjectHolder, layer);
            }
            layers.clear();
        }

        /**
         * call update on each layer
         */
        public function update(deltaTime:Number):void
        {
            for (var node:LinkedListNode = layers.first; node!=null; node=node.next)
            {
                var layer:GameObjectHolder = node.value;
                layer.update(deltaTime);
            }
        }

        /**
         * call integrate on each layer
         */
        public function integrate(deltaTime:Number):void
        {
            for (var node:LinkedListNode = layers.first; node!=null; node=node.next)
            {
                var layer:GameObjectHolder  = node.value;
                layer.integrate(deltaTime);
            }
        }

        /**
         * call preCollision on each layer
         */
        public function preCollision(deltaTime:Number):void
        {
            for (var node:LinkedListNode = layers.first; node!=null; node=node.next)
            {
                var layer:GameObjectHolder  = node.value;
                layer.preCollision(deltaTime);
            }
        }

        /**
         * call collision on each layer
         */
        public function collision(deltaTime:Number):void
        {
            for (var node:LinkedListNode = layers.first; node!=null; node=node.next)
            {
                var layer:GameObjectHolder  = node.value;
                layer.collision(deltaTime);
            }
        }

        /**
         * call after all the final positions have been figured
         * just before drawing.
         * Calls preDraw on each layer.
         */
        public function preDraw(deltaTime:Number):void
        {
            for (var node:LinkedListNode = layers.first; node!=null; node=node.next)
            {
                var layer:GameObjectHolder  = node.value;
                layer.preDraw(deltaTime);
            }
        }

        /**
         * needs to be called once so that we can attach
         * the layers to the right container
         */
        public function init(container:DisplayObjectContainer, game:GameInterface):void
        {
            layers = new LinkedList();
            this.container = container;
            this.game = game;
        }


        /**
         * create a layer and add it to the scene graph and layer list
         */
        public function addLayer():GameObjectHolder
        {
            var layer:Sprite = ObjectPool.getObject(Sprite);
            GraphicsUtil.clearSprite(layer);
            container.addChild(layer);
            var goh:GameObjectHolder  = ObjectPool.getObject(GameObjectHolder );
            goh.removeAll();
            goh.container = layer;
            goh.game = game;
            layers.push_back(goh);
            return goh;
        }

        /**
         * clean up anything that htis could possibly leave dangling
         */
        public function shutdown():void
        {
            removeAll();
            layers = null;
        }
    }
}
