﻿package stego.util.schedule
{
    import stego.script.Step;
    import stego.util.ArrayUtil;

    /**
     * keep track of things that are supposed to happen in the future and make
     * sure they happen when they are supposed to
     * @author Mike Meyer
     */
    public class Schedule
    {
        protected var scheduledItems:Array = [];

        public function callEvery(delay:Number, callback:Function):void
        {
            scheduledItems.push(new ScheduleItem(delay, callback, true));
        }

        public function callOnce(delay:Number, callback:Function):void
        {
            scheduledItems.push(new ScheduleItem(delay, callback, false));
        }

        public function runScript(step:Step):void
        {
            scheduledItems.push(new ScriptScheduleItem(step));
        }

        public function watchCountdown(delay:Number, callback:Function):void
        {
            scheduledItems.push(new CountdownScheduleItem(delay, callback));
        }

        /**
         * remove a function from the schedule
         */
        public function cancel(callback:Function):void
        {
            var predicate:Function = function(scheduleItem:IScheduleItem):Boolean { return (scheduleItem.callback == callback); };
            ArrayUtil.removePredicate(scheduledItems, predicate );
        }

        /**
         * per-frame update
         * @param	deltaTime - time since the last update (in seconds)
         */
        public function update(deltaTime:Number):void
        {
            for (var i:int = scheduledItems.length - 1; i >= 0 ; --i)
            {
                var scheduleItem:IScheduleItem = scheduledItems[i] as IScheduleItem;
                scheduleItem.update(deltaTime);
                if (scheduleItem.isDone)
                {
                    scheduledItems.splice(i, 1);
                }
            }
        }

        /**
         * clean up
         */
        public function shutdown():void
        {
            scheduledItems = [];
        }
    }

}
