﻿package stego.util.schedule
{
    /**
     * ...
     * @author Mike Meyer
     */
    public class ScheduleItem implements IScheduleItem
    {
        /**
         * how long to wait between calls
         */
        protected var delay:Number;

        /**
         * function to call after the delay
         * should not require any arguments
         */
        public function get callback():Function { return _callback; }
        private var _callback:Function;

        /**
         * countdown to the next call
         */
        protected var timeToNext:Number;

        /**
         * whether to repeat the delay+call or to stop forever after caling it
         * once
         */
        protected var repeat:Boolean;

        /**
         * Will this schedule item ever run again?
         */
        public function get isDone():Boolean { return (!repeat && (timeToNext <= 0)); }

        /**
         * constructor
         * @param	delay - how long to wait between calls
         * @param	callback - function to call
         * @param	repeat - whether to repeat the function call or stop after one
         */
        public function ScheduleItem(delay:Number, callback:Function, repeat:Boolean)
        {
            this.delay = delay;
            this._callback = callback;
            this.timeToNext = delay;
            this.repeat = repeat;
        }

        /**
         * tick the countdown, call the callback if it is time for it
         * @param	deltaTime - how much time has elapsed since the last
         * call to Update
         */
        public function update(deltaTime:Number):void
        {
            if (!isDone)
            {
                timeToNext -= deltaTime;

                if (timeToNext <= 0)
                {
                    callback();

                    if (repeat)
                    {
                        timeToNext = delay;
                    }
                    else
                    {
                        _callback = null;
                    }
                }
            }
        }
    }

}
