﻿package stego.util.spriteanim
{
    import flash.display.Bitmap;
    import flash.display.DisplayObject;
    import flash.display.DisplayObjectContainer;
    import flash.display.Sprite;
    import flash.utils.Dictionary;

    /**
     * ...
     * @author Mike Meyer
     */
    public class SpriteAnimUtil
    {
        private var holder:DisplayObjectContainer;
        private var animations:Object;

        private var _curAnim:String;
        /**
         * get/set the animation to use
         */
        public function get curAnim():String { return _curAnim; }
        public function set curAnim(value:String):void
        {
            if (animations.hasOwnProperty(_curAnim))
            {
                animations[_curAnim].visible = false;
            }
            _curAnim = value;
            if (animations.hasOwnProperty(_curAnim))
            {
                animations[_curAnim].visible = true;
            }
        }

        /// Indicates the horizontal scale (percentage) of an object as applied from the registration point of the object.
        public function get scaleX():Number { return holder.scaleX; }
        public function set scaleX(value:Number):void { holder.scaleX = value; }

        /// Indicates the vertical scale (percentage) of an object as applied from the registration point of the object.
        public function get scaleY():Number { return holder.scaleY; }
        public function set scaleY(value:Number):void { holder.scaleY = value; }

        /**
         * make a new sprite anim util
         * @param	parent DisplayObjectContainer in which the animation frames should live
         */
        public function SpriteAnimUtil(parent:DisplayObjectContainer)
        {
            holder = new Sprite();
            animations = new Dictionary();
            parent.addChild(holder);
            holder.scaleX = scaleX;
            holder.scaleY = scaleY;
        }

        /**
         * add an animation to the utility
         * @param	name - name you will be using to start the animation
         * @param	BitMapClasses - array of classes to be instantiated
         * @param   framerate - how many frames of animation play in a second
         * @param   loop - whether to loop the animation. defaults to true
         * as frames of the animation
         */
        public function addAnimation(name:String, bitMapClasses:Array, framerate:Number = 8.0, loop:Boolean = true):void
        {
            var spriteanim:SpriteAnim = new SpriteAnim();
            spriteanim.framerate = framerate;
            spriteanim.frames = bitMapClasses.map(instantiateFrame);
            spriteanim.loop = loop;
            animations[name] = spriteanim;
            spriteanim.visible = (name == _curAnim);
        }

        /**
         * instantiate the class, add it to the holder, return it
         * @param	bitMapClass - class to instantiate to get the frame graphic
         */
        private function instantiateFrame(bitMapClass:Class, index:int, arr:Array):DisplayObject
        {
            var bitmap:Bitmap = new bitMapClass();
            bitmap.visible = false;
            bitmap.x = -0.5 * bitmap.width;
            bitmap.y = -0.5 * bitmap.height;
            bitmap.smoothing = false;
            holder.addChild(bitmap);
            return bitmap;
        }

        /**
         * call every frame to make the animations go
         * @param	deltaTime - time since the last call
         */
        public function update(deltaTime:Number):void
        {
            if (animations.hasOwnProperty(curAnim) == true)
            {
                var spriteanim:SpriteAnim = animations[curAnim];
                spriteanim.update(deltaTime);
            }
            else if ( (null != curAnim) && ("" != curAnim) )
            {
                throw new Error("Missing animation! \"" + curAnim + "\"");
            }
        }
    }

}
